*                                    AVERTISSEMENT

* Le gouvernement du Canada (Statistique Canada) possde tous les droits de proprit
* intellectuelle (y compris le droit d'auteur) relatifs  ce logiciel.  Sous rserve
* des modalits dcrites ci-dessous, vous obtenez une licence non exclusive et non
* transmissible d'utilisation de ce logiciel.
*
* Ce logiciel vous est fourni  tel quel , et le titulaire ne donne aucune garantie
* expresse ou implicite, ce qui comprend les garanties de qualit marchande et de
* convenance  une fin particulire mais ne se limite pas  ces garanties.  En aucune
* circonstance le titulaire ne sera responsable de dommages indirects, particuliers ou
* conscutifs, ou autres dommages semblables.  Ce contrat sera automatiquement rsili,
* sans pravis, si vous omettez de respecter une quelconque des modalits du contrat.;

/*******************************************************************/
/* Date: Janvier 2005                                              */
/*******************************************************************/

/*********************************************************************************/
/***                                                                           ***/
/***                             MACROF_V31.SAS                                ***/
/***                                                                           ***/
/*** Ce programme calcule des estimations de variance pour diffrents types    ***/
/*** d'estimations  l'aide des poids bootstrap.   l'aide de macros SAS,      ***/
/*** ce programme peut calculer des estimations de variance pour des totaux,   ***/
/*** des ratios, et des diffrences de ratios.  Il peut aussi calculer des     ***/
/*** estimations de variance pour des paramtres de rgressions linaires ou   ***/
/*** logistiques.  Ce programme peut aussi tre modifi pour s'adapter        ***/
/*** d'autres types d'analyse.                                                 ***/
/***                                                                           ***/
/*** Ce programme contient les macros qui sont ncessaires pour utiliser le    ***/
/*** programme BOOTVARF_V31.SAS.                                               ***/
/***                                                                           ***/
/*** Ce programme est appel automatiquement par BOOTVARF_V31.SAS et AUCUNE    ***/
/*** MODIFICATION NE DEVRAIT ETRE APPORTE PAR L'UTILISATEUR (sauf pour        ***/
/*** quelques cas prcis mentionns dans la documentation)                     ***/
/***                                                                           ***/
/*********************************************************************************/

options ls=120 nonotes;


*********************************************************************************************
*** Section 1: DCLARATION DES VARIABLES MARCRO (PEUT TRE MODIFI PAR LES UTILISATEURS)  ***
*********************************************************************************************;

	%let deff=0;        * Active le calcul des effets de plan (total et ratio);
	%let alpha=0.05;    * Permet de modifier le niveau de confiance lors du calcul des intervalles de confiance;


**********************************************************************************************************
***  Section 2: CRATION DES VARIABLES MACRO UTILISES PAR LE PROGRAMME (NE DEVRAIT PAS TRE MODIFI)  ***
**********************************************************************************************************;

* VRIFICATION SI UNE (DES) VARIABLE(S) DE VENTILATION EST (SONT) SPCIFIES; 

		%let by=;
		%let cla_tmp="&classes";

		data _NULL_;
			if substr(&cla_tmp,1,1)='.' then do; 
         									call symput ('by' ,'*');
         									call symput ('classes','');
         									call symput ('number',0);
										end;
		run;


* POUR OBTENIR LES MARGINALES DANS LES RSULTATS (On spare la variable macro classes en plusieurs variables);

		&by  %let cla_tmp="&classes"||' #';

		&by  data _NULL_;
		&by  do i=1 to 10;
		&by  	call symput ('cla'||left(trim(i)),' ');
		&by  end;
		&by  init=1; &by  i=1; &by  fin=1; &by  stop=' ';
		&by  do until (stop='#');
		&by  	do until (substr(&cla_tmp,i,1)='');
		&by  		call symput ('cla'||left(trim(init)),substr(&cla_tmp,fin,i-fin+1));
		&by  		call symput ('number',init);
		&by  		i=i+1;
		&by  	end;
		&by  	fin=i;
		&by  	stop=substr(&cla_tmp,i+1,1);
		&by  	init=init+1;
		&by  end;
		&by  run;


* UNIFORMISATION DU NOM DES VARIABLES SPCIFIQUES A CHAQUE VERSION DE SAS ;

		data _NULL_;
			if &sysver >= 8 then call symput ('inter' ,'intercept');
			else call symput ('inter' ,'intercep');
		run;


* VARIABLE NCESSAIRE POUR INDEX (section suivante) ;

		%let indx=(id=(&ident blank)); 


OPTIONS notes;



*******************************************************************************
***  Section 3: LECTURE DU FICHIER PRINCIPAL ET APPARIEMENT AVEC LES POIDS  ***
*******************************************************************************;

		data Mfile (index=&indx);
		   set &Mfile ;
		   blank=.;
		run;

*** Lire les poids bootstrap et standardiser le nom de chaque variable de poids  ***;

		OPTIONS nonotes;

		data bsamp (index=&indx);
		  set &bsamp;
		   blank=.;
		         if &bsw.1 ne '' then do; 
		                           call symput ('bsw_frst' ,'&bsw.1');
		                           call symput ('bsw_last' ,'&bsw.&b');
								end;

		         else if &bsw.001 ne '' then do; 
		                           call symput ('bsw_frst' ,'&bsw.001');
		                           call symput ('bsw_last' ,'&bsw.%sysfunc(putn(&b,z3.))');
								end;
		         else if &bsw.0001 ne '' then do; 
		                           call symput ('bsw_frst' ,'&bsw.0001');
		                           call symput ('bsw_last' ,'&bsw.%sysfunc(putn(&b,z4.))');
								end;   
		run;

		OPTIONS notes;


*** Apparie le fichier Analyse avec les poids bootstrap ***;

	     data bs_data ;
	           merge Mfile (in=in1) bsamp (keep= &ident &fwgt &bsw_frst-&bsw_last);
	           by &ident;
	           if in1;
		       drop blank;
	           rename &bsw_frst-&bsw_last=bsw1-bsw&b; 
			   run;


		&by  proc sort data=bs_data;
		&by     by &classes;
		&by  run;


*** Initialise les fichiers de rsultats   ***;

		data alltots allrats diffrat allperc bs_reg bs_reglg allchi2;
		      set _NULL_;
		run;

		%let result= alltots allrats diffrat allperc bs_reg bs_reglg allchi2;


*********************************************************************************;
*** Section 4: DCLARATION DES MACROS                                         ***;
*********************************************************************************;

*** Initialisation de variables macros utilises dans les macros ***;

	%let printtot=0;
	%let printrat=0;
	%let printdif=0;
	%let printperc=0;
	%let printreg=0;
	%let printlog=0;
	%let printchi2=0;

	%let model_reg=0;
	%let model_log=0;
	%let test_chi2=0;
	%let test_num2=0;
	%let flag_rat=0;
	%let flag_dif=0;
	%let flag_perc=0;
	%let flag_reg=0;
	%let flag_log=0;
	%let ic_inf=0;
	%let ic_sup=0;
	%let ic=0;

************************************************
****                                        ****
****                 TOTAL                  ****
****                                        ****
************************************************;


%macro total(var);

*** Dterminer si des valeurs ngatives sont prsentes ***;

		%let negative=0;
		data _NULL_;
			set bs_data (keep=&var);
			if &var ne 0 and &var / abs(&var)=-1  then do; call symput('negative',1); end;
		run;

*** Si aucune valeur ngative prsente, utiliser la mthode rapide ***;

	%if (&negative=0) %then %do;
		proc means data=bs_data noprint;
		    var &fwgt bsw1-bsw&b;
		    weight &var;
		&by  class &classes;
		    output out=ytot
		         sum=yhat ybs1-ybs&b;
		run;
	%end;

*** Si valeurs ngatives prsentes, utiliser la mthode longue ***;

	%if (&negative=1) %then %do;
		OPTIONS nonotes;
		proc means data=bs_data noprint;
		    var &var;
		    weight &fwgt;
		&by  class &classes;
		  output out=ytot sum=yhat;
		run;

		&by proc sort data=ytot;
		&by 	by &classes;
		&by run;

		%do looper=1 %to &B;

			proc means data=bs_data noprint;
		  		var &var;
		  		weight bsw&looper;
			&by  class &classes;
		    output out=ytot_bsw sum=ybs&looper;
			run;

			&by proc sort data=ytot_bsw;
			&by 	by &classes;
			&by	run;
			
			data ytot;	
				merge ytot ytot_bsw;
				&by by &classes;
			run;
		%end;
		OPTIONS notes;
	%end;

*** Calcul - taille chantillon  ***;

		proc means data=Mfile noprint;
		    var &var;
		    where &var>0;
		&by  class &classes;
		    output out=n n=n;
		run;

		&by proc sort data=ytot;
		&by 	by &classes;
		&by run;

		&by proc sort data=n;
		&by 	by &classes;
		&by run;

		data ytot;
			merge ytot n (drop=_type_ _freq_);
		&by by &classes;
			if (&negative=1) then n=.;   * annuler le calcul de n si valeurs ngatives pour viter de mauvaises interprtations;
		run;

*** Variables pour le niveau de confiance ;

		%let niveau=probit(1-(&alpha/2)); 
		data _NULL_;
			call symput ('ic' ,(1-&alpha)*100);
		run;
		data _NULL_;
	 		call symput ('ic_inf',compress("ic_i&ic"));
	 		call symput ('ic_sup',compress("ic_s&ic"));
		run;

*** Calcul des rsultats ***;

		data est;
			  set ytot;
			  length var $ 8;
			  length type $ 8;
			  Estimation=yhat;
			  bs_var=&r*((&b-1)*(var(of ybs1-ybs&b)))/&b;
			  bs_et=sqrt(bs_var);
			  if Estimation ne 0 then bs_cv=abs(round((bs_et/Estimation)*100,.01));
			  else bs_cv=.;
			  &ic_inf=Estimation-&niveau*bs_et;
			  &ic_sup=Estimation+&niveau*bs_et;
			  var="&var";
			  type="Total";
			  drop ybs1-ybs&b yhat _type_ _freq_;
		run;

*** Calcul Effet de plan (si activ au dbut du programme)***;

		%if (&deff=1) %then %do;
			proc means data=bs_data noprint;
			    var &fwgt;
			&by  class &classes;
			  output out=deff1 n=nsamp sum=npop;

			proc means data=bs_data noprint vardef=wdf;
			    var &var;
		        weight &fwgt;
			&by  class &classes;
			  output out=deff2 var=yvar;

			&by proc sort data=deff1;
			&by 	by &classes;
			&by run;
			&by proc sort data=deff2;
			&by 	by &classes;
			&by run;

			data est;
			    merge est deff1 deff2;
			    &by 	by &classes;
 			    vsrs=npop*npop*yvar/nsamp; 
			    deff=bs_var/vsrs; 
			    drop npop nsamp yvar vsrs _type_ _freq_; 

			proc datasets library=work;
			    delete deff1 deff2;
			run;
		%end;

*** Fin - Calcul Effet de plan ***;

*** Cration du fichier de rsultats ***;

		&by   data est;
		&by      set est;
		&by  			%do k=1 %to 10;
		&by  			cla&k='        ';
		&by  			%end;
		&by  			%do k=1 %to &number; 
		&by  			cla&k=input(compress(&&cla&k),$8.);
		&by  			%end;
		&by   run;

		data alltots;
			set alltots est;
		run;

		%let printtot=1;

		proc datasets library=work;
			delete ytot est n;
		run;


%mend total;



************************************************
****                                        ****
****                 RATIO                  ****
****                                        ****
************************************************;


%macro ratio(var1,var2);

*** Dterminer si des valeurs ngatives sont prsentes ***;

		%let negative=0;
		data _NULL_;
			set bs_data (keep=&var1 &var2 );
			if &var1 ne 0 and &var1 / abs(&var1)=-1  then do; call symput('negative',1); end;
			if &var2 ne 0 and &var2 / abs(&var2)=-1  then do; call symput('negative',1); end;
		run;

*** Si aucune valeur ngative prsente, utiliser la mthode rapide ***;

	%if (&negative=0) %then %do;
		proc means data=bs_data noprint;
		  var &fwgt bsw1-bsw&b;
		  weight &var1;
		&by  class &classes;
		  output out=ytot
		         sum=yhat ybs1-ybs&b;
		run;

		proc means data=bs_data noprint;
		  var &fwgt bsw1-bsw&b;
		  weight &var2;
		  &by  class &classes;
		  output out=xtot
		  sum=xhat xbs1-xbs&b;
		run;
	%end;


*** Si valeurs ngatives prsentes, utiliser la mthode longue ***;

	%if (&negative=1) %then %do;
		OPTIONS nonotes;
		proc means data=bs_data noprint;
		  var &var1;
		  weight &fwgt;
		&by  class &classes;
		  output out=ytot sum=yhat;
		run;

		&by proc sort data=ytot;
		&by 	by &classes;
		&by run;

		%do looper=1 %to &B;

			proc means data=bs_data noprint;
		  		var &var1;
		  		weight bsw&looper;
			&by  class &classes;
		    output out=ytot_bsw sum=ybs&looper;
			run;

			&by proc sort data=ytot_bsw;
			&by 	by &classes;
			&by	run;
			
			data ytot;	
				merge ytot ytot_bsw;
				&by by &classes;
			run;
		%end;

		proc means data=bs_data noprint;
		  var &var2;
		  weight &fwgt;
		&by  class &classes;
		  output out=xtot sum=xhat;
		run;

		&by proc sort data=xtot;
		&by 	by &classes;
		&by run;

		%do looper=1 %to &B;

			proc means data=bs_data noprint;
		  		var &var2;
		  		weight bsw&looper;
			&by  class &classes;
		    output out=xtot_bsw sum=xbs&looper;
			run;

			&by proc sort data=xtot_bsw;
			&by 	by &classes;
			&by	run;
			
			data xtot;	
				merge xtot xtot_bsw;
				&by by &classes;
			run;
		%end;

		OPTIONS notes;
	%end;


*** Calcul - taille chantillon ***;

		proc means data=Mfile noprint;
		     var &var1;
		     where &var1>0;
		&by  class &classes;
		     output out=n n=n1;
		run;

		&by proc sort data=ytot;
		&by 	by &classes;
		&by run;

		&by proc sort data=xtot;
		&by 	by &classes;
		&by run;

		&by proc sort data=n;
		&by 	by &classes;
		&by run;

		data ytot;
		 	 merge ytot n (drop=_type_ _freq_);
		&by by  &classes;
			if (&negative=1) then n1=.;   * annuler le calcul de n si valeurs ngatives pour viter de mauvaises interprtations;
		run;

*** Variables pour le niveau de confiance ;

		%let niveau=probit(1-(&alpha/2)); 
		data _NULL_;
			call symput ('ic' ,(1-&alpha)*100);
		run;
		data _NULL_;
	 		call symput ('ic_inf',compress("ic_i&ic"));
	 		call symput ('ic_sup',compress("ic_s&ic"));
		run;

*** Calcul des rsultats ***;

		data est;
			  merge ytot xtot;
		      &by by  &classes;
  			  array ybs{&b};
			  array xbs{&b};
			  array rbs{&b};
			  length var1 $ 8;
			  length var2 $ 8;
			  length type $ 8;
			  if xhat ne 0 then Estimation=((yhat/xhat));
			  rep_mod=0;
			  do i=1 to &b;
			       if xbs{i} ne 0 then rbs{i}=((ybs{i}/xbs{i}));
				   *** Calcul du nombre de rpliques non-vides ***;
				   if xbs{i} ne 0 and xbs{i} ne . then rep_mod=rep_mod+1;  
			  end;
			  if rep_mod ne 0 then bs_var=&r*((rep_mod-1)*(var(of rbs1-rbs&b)))/rep_mod;
			  bs_et=sqrt(bs_var);
			  if estimation ne 0 then bs_cv=abs(round((bs_et/estimation)*100,.01));
			  else bs_cv=.;
			  &ic_inf=Estimation-&niveau*bs_et;
			  &ic_sup=Estimation+&niveau*bs_et;
			  var1="&var1";
			  var2="&var2";
			  type="Ratio";
			  if rep_mod ne &b then call symput('flag_rat',1);
			  drop ybs1-ybs&b xbs1-xbs&b rbs1-rbs&b xhat yhat i _type_ _freq_;
		run;

*** Calcul Effet de plan (si activ au dbut du programme) ***;

		%if (&deff=1) %then %do;
			proc means data=bs_data noprint vardef=wdf;
			  var &var1 &var2;
			  weight &fwgt;
			&by  by &classes;
			  output out=deff1 n=nsamp mean=ybar xbar var=yvar xvar;

			proc corr data=bs_data cov vardef=wdf outp=deff2 noprint;
			  var &var1 &var2;
			  weight &fwgt;
			&by  by &classes;

			data deff2;
			  set deff2;
			  where _type_='COV' and upcase(_name_)=left(upcase("&var1"));
			  rename &var2=cov;
			  drop _name_ &var1 _type_;

			&by proc sort data=deff1;
			&by 	by &classes;
			&by run;
			&by proc sort data=deff2;
			&by 	by &classes;
			&by run;

			data est;
			  merge est deff1 deff2 (in=in1);
			&by  by &classes;
		 	  if in1;
			  
			  if xbar not in (0,.) and ybar not in (0,.) then do;
			     vsrs=(1/nsamp)*(ybar**2)/(xbar**2)*
			          (yvar/ybar**2+xvar/xbar**2-2*cov/xbar/ybar);
			     deff=bs_var/vsrs;
			  end;
			 drop nsamp ybar xbar yvar xvar cov vsrs _type_ _freq_; 

			proc datasets library=work;
			  delete deff1 deff2;
			run;
		%end;

*** Fin - Calcul Effet de plan  ***;

*** Cration du fichier de rsultats ***;

		&by   data est;
		&by      set est;
		&by  			%do k=1 %to 10;
		&by  			cla&k='        ';
		&by  			%end;
		&by  			%do k=1 %to &number; 
		&by  			cla&k=input(compress(&&cla&k),$8.);
		&by  			%end;
		&by   run;

		data allrats;
			set allrats est;
		run;

		%let printrat=1;

		proc datasets library=work;
		    delete ytot xtot ytot_bsw xtot_bsw est n;
		run;

%mend ratio;



************************************************
****                                        ****
****      DIFFRENCE ENTRE RATIOS           ****
****                                        ****
************************************************;


%macro diff_rat(var1,var2,var3,var4);

*** Dterminer si des valeurs ngatives sont prsentes ***;

		%let negative=0;
		data _NULL_;
			set bs_data (keep=&var1 &var2 &var3 &var4);
			if &var1 ne 0 and &var1 / abs(&var1)=-1  then do; call symput('negative',1); end;
			if &var2 ne 0 and &var2 / abs(&var2)=-1  then do; call symput('negative',1); end;
			if &var3 ne 0 and &var3 / abs(&var3)=-1  then do; call symput('negative',1); end;
			if &var4 ne 0 and &var4 / abs(&var4)=-1  then do; call symput('negative',1); end;
		run;

*** Si aucune valeur ngative prsente, utiliser la mthode rapide ***;

	%if (&negative=0) %then %do;
		proc means data=bs_data noprint;
		  var &fwgt bsw1-bsw&b;
		  weight &var1;
		  &by  class &classes;
		  output out=ytot
		  sum=yhat ybs1-ybs&b;
		run;

		proc means data=bs_data noprint;
		  var &fwgt bsw1-bsw&b;
		  weight &var2;
		  &by  class &classes;
		  output out=xtot
		  sum=xhat xbs1-xbs&b;
		run;

		proc means data=bs_data noprint;
		  var &fwgt bsw1-bsw&b;
		  weight &var3;
		  &by  class &classes;
		  output out=yytot
		  sum=yyhat yybs1-yybs&b;
		run;

		proc means data=bs_data noprint;
		  var &fwgt bsw1-bsw&b;
		  weight &var4;
		  &by  class &classes;
		  output out=xxtot
		  sum=xxhat xxbs1-xxbs&b;
		run;
	%end;

*** Si valeurs ngatives prsentes, utiliser la mthode longue ***;

	%if (&negative=1) %then %do;

		OPTIONS nonotes;

		proc means data=bs_data noprint;
		     var &var1;
		     weight &fwgt;
		&by  class &classes;
		     output out=ytot sum=yhat;
		run;

		&by proc sort data=ytot;
		&by 	by &classes;
		&by run;

		%do looper=1 %to &B;

			proc means data=bs_data noprint;
		  		var &var1;
		  		weight bsw&looper;
			&by  class &classes;
		    output out=ytot_bsw sum=ybs&looper;
			run;

			&by proc sort data=ytot_bsw;
			&by 	by &classes;
			&by	run;
			
			data ytot;	
				merge ytot ytot_bsw;
				&by by &classes;
			run;
		%end;

		proc means data=bs_data noprint;
		     var &var2;
		     weight &fwgt;
		&by  class &classes;
		  output out=xtot sum=xhat;
		run;

		&by proc sort data=xtot;
		&by 	by &classes;
		&by run;

		%do looper=1 %to &B;

			proc means data=bs_data noprint;
		  		var &var2;
		  		weight bsw&looper;
			&by  class &classes;
		    output out=xtot_bsw sum=xbs&looper;
			run;

			&by proc sort data=xtot_bsw;
			&by 	by &classes;
			&by	run;
			
			data xtot;	
				merge xtot xtot_bsw;
				&by by &classes;
			run;
		%end;

		proc means data=bs_data noprint;
		     var &var3;
		     weight &fwgt;
		&by  class &classes;
		  output out=yytot sum=yyhat;
		run;

		&by proc sort data=yytot;
		&by 	by &classes;
		&by run;

		%do looper=1 %to &B;

			proc means data=bs_data noprint;
		  		var &var3;
		  		weight bsw&looper;
			&by  class &classes;
		    output out=yytot_bsw sum=yybs&looper;
			run;

			&by proc sort data=yytot_bsw;
			&by 	by &classes;
			&by	run;
			
			data yytot;	
				merge yytot yytot_bsw;
				&by by &classes;
			run;
		%end;

		proc means data=bs_data noprint;
		     var &var4;
		     weight &fwgt;
		&by  class &classes;
		  output out=xxtot sum=xxhat;
		run;

		&by proc sort data=xxtot;
		&by 	by &classes;
		&by run;

		%do looper=1 %to &B;

			proc means data=bs_data noprint;
		  		var &var4;
		  		weight bsw&looper;
			&by  class &classes;
		    output out=xxtot_bsw sum=xxbs&looper;
			run;

			&by proc sort data=xxtot_bsw;
			&by 	by &classes;
			&by	run;
			
			data xxtot;	
				merge xxtot xxtot_bsw;
				&by by &classes;
			run;
		%end;
		OPTIONS notes;
	%end;

*** Calcul - taille chantillon ***;

		proc means data=Mfile noprint;
		     var &var1;
		     where &var1>0;
		&by  class &classes;
		  output out=n1 n=n1;
		run;

		proc means data=Mfile noprint;
		     var &var3;
		     where &var3>0;
		&by  class &classes;
		     output out=n3 n=n3;
		run;

		&by proc sort data=ytot;
		&by 	by &classes;
		&by run;
		&by proc sort data=xtot;
		&by 	by &classes;
		&by run;
		&by proc sort data=yytot;
		&by 	by &classes;
		&by run;
		&by proc sort data=xxtot;
		&by 	by &classes;
		&by run;
		&by proc sort data=n1;
		&by 	by &classes;
		&by run;
		&by proc sort data=n3;
		&by 	by &classes;
		&by run;

		data ytot;
			merge ytot n1 (drop=_type_ _freq_) n3 (drop=_type_ _freq_);
		&by by &classes;
			if (&negative=1) then do; n1=.; n3=.; end;  * annuler le calcul de n si valeurs ngatives pour viter de mauvaises interprtations;
		run;

*** Variables pour le niveau de confiance ;

		%let niveau=probit(1-(&alpha/2)); 
		data _NULL_;
			call symput ('ic' ,(1-&alpha)*100);
		run;
		data _NULL_;
	 		call symput ('ic_inf',compress("ic_i&ic"));
	 		call symput ('ic_sup',compress("ic_s&ic"));
		run;

*** Calcul des rsultats ***;

	  data est;
		  merge ytot xtot yytot xxtot;
		  &by 	by &classes;
		  array ybs{&b};
		  array xbs{&b};
		  array yybs{&b};
		  array xxbs{&b};
		  array drbs{&b};
		  length var1 $ 8;
		  length var2 $ 8;
		  length var3 $ 8;
		  length var4 $ 8;
		  length type $ 10;
		  if xhat ne 0 and xxhat ne 0 then Estimation=(((yhat/xhat)-(yyhat/xxhat)));
		  rep_mod=0;
		  do i=1 to &b;
		    if xbs{i} ne 0 and xxbs{i} ne 0 then drbs{i}=(((ybs{i}/xbs{i})-(yybs{i}/xxbs{i})));
			*** Calcul du nombre de rpliques non-vides ***;
		    if xbs{i} ne 0 and xbs{i} ne . and xxbs{i} ne 0 and xxbs{i} ne . then rep_mod=rep_mod+1;  
		  end;
		  if rep_mod ne 0 then bs_var=&r*(((rep_mod-1)*(var(of drbs1-drbs&b)))/rep_mod);
		  bs_et=sqrt(bs_var);
		  if Estimation ne 0 then bs_cv=abs(round((bs_et/Estimation)*100,.01));
		  else bs_cv=.;
		  if bs_et ne 0 then z=estimation/bs_et;
		  valeurp=2*(1-probnorm(abs(z)));
		  &ic_inf=Estimation-&niveau*bs_et;
		  &ic_sup=Estimation+&niveau*bs_et;
		  var1="&var1";
		  var2="&var2";
		  var3="&var3";
		  var4="&var4";
		  type="Dif_Rat";
		  if rep_mod ne &b then call symput('flag_dif',1);
		  drop ybs1-ybs&b xbs1-xbs&b yybs1-yybs&b xxbs1-xxbs&b drbs1-drbs&b
		       xhat yhat xxhat yyhat i _type_ _freq_;

		  &by  			%do k=1 %to 10;
		  &by  			cla&k='        ';
		  &by  			%end;
		  &by  			%do k=1 %to &number; 
		  &by  			cla&k=input(compress(&&cla&k),$8.);
		  &by  			%end;
		  run;


*** Cration du fichier de rsultats ***;

	  	data diffrat;
			set diffrat est;
	    run;

		%let printdif=1;

		proc datasets library=work;
		    delete ytot xtot yytot xxtot est n1 n3;
		run;


%mend diff_rat;



************************************************
****                                        ****
****             PERCENTILE                 ****
****                                        ****
************************************************;


%macro prcntle(var,perc);

*** Calcul des percentiles ***;

		proc univariate data=bs_data noprint;
		     var &var;
		     weight &fwgt;
		&by  by &classes;
		     output out=ytot pctlpts=&perc pctlpre=_ pctlname=P&perc._yhat;
		run;

		&by proc sort data=ytot;
		&by 	by &classes;
		&by run;

		options nonotes;

		%do looper=1 %to &B;

			proc univariate data=bs_data noprint;
		  		var &var;
		  		weight bsw&looper;
			&by  by &classes;
		    output out=pctl pctlpts=&perc pctlpre=_ pctlname=P&perc._ybs&looper;
			run;

			&by proc sort data=pctl;
			&by 	by &classes;
			&by	run;
			
			data ytot;	
				merge ytot(in=a) pctl(in=b);
				&by by &classes;
				if a and b;
				label _P&perc._yhat='yhat';
			run;

		%end;

		options notes;

*** Calcul - taille chantillon ***;

		proc means data=Mfile noprint;
		    var &var;
		    where &var>0;
		&by  class &classes;
		    output out=n n=n;
		run;

		&by proc sort data=ytot;
		&by 	by &classes;
		&by run;

		&by proc sort data=n;
		&by 	by &classes;
		&by run;

		data ytot;
			merge ytot (in=a) n (drop=_type_ _freq_);
		&by by &classes;
			if a;
		run;

*** Identification des rpliques non-vides (les ybs sont recods  manquant)***;

		proc means data=bs_data noprint;
		    var bsw1-bsw&b;
		&by  by &classes;
		    output out=ytot_nul
		         sum= ybs_nul1-ybs_nul&b;
		run;

		data ytot (drop=ybs_nul1-ybs_nul&b);
			merge ytot ytot_nul;
		&by  by &classes;
			array _P&perc._ybs{&b};
			array ybs_nul{&b};
			do i = 1 to &b;
				if ybs_nul{i}=0 or ybs_nul{i}=. then _P&perc._ybs{i}=.;
			end;
		run;

*** Variables pour le niveau de confiance ;

		%let niveau=probit(1-(&alpha/2)); 
		data _NULL_;
			call symput ('ic' ,(1-&alpha)*100);
		run;
		data _NULL_;
	 		call symput ('ic_inf',compress("ic_i&ic"));
	 		call symput ('ic_sup',compress("ic_s&ic"));
		run;

*** Calcul des rsultats ***;

	   data est;
		  set ytot(rename=(_P&perc._yhat=yhat _P&perc._ybs1-_P&perc._ybs&b=ybs1-ybs&b));
	  	  array ybs{&b};
		  length var $ 8;
		  length perc $ 8;
		  length type $ 8;
		  Estimation=yhat;
		  	*** Calcul du nombre de rpliques non-vides ***;
		  	rep_mod=0;
		    do i=1 to &b;
		  	if ybs{i} ne . then rep_mod=rep_mod+1;  
		    end;
		  if rep_mod ne 0 then bs_var=&r*((rep_mod-1)*(var(of ybs1-ybs&b)))/rep_mod;
		  bs_et=sqrt(bs_var);
		  if Estimation ne 0 then bs_cv=round((bs_et/Estimation)*100,.01);
		  else bs_cv=.;
		  &ic_inf=Estimation-&niveau*bs_et;
		  &ic_sup=Estimation+&niveau*bs_et;
		  var="&var";
		  perc="&perc";
		  type="Prcntle";
		  if rep_mod ne &b then call symput('flag_perc',1);
		  drop ybs1-ybs&b yhat i ;
	   run;

*** Cration du fichier de rsultats ***;

		%let printperc=1;

		data allperc;
		   set allperc est;
		run;

		proc datasets library=work;
		   delete ytot ytot_nul pctl est n;
		run;


%mend prcntle;



************************************************
****                                        ****
****         RGRESSION LINAIRE            ****
****                                        ****
************************************************;


%macro regress(yvar,xvar);

		%let model_reg=&model_reg+1;
		OPTIONS nonotes;

*** Estimations ponctuelles ***;

		proc reg data=bs_data outest=orig(keep=&classes &inter &xvar) noprint;
		  model &yvar=&xvar;
		  weight &fwgt;
		  &by  by &classes;
		run;

		proc transpose data=orig out=origest(drop=_label_) prefix=beta name=param;
		  var &inter &xvar;
		  &by  by &classes;
		run;

*** Calcul de la variance bootstrap ***;

		data _NULL_; 
		    L=int((&b/10)+0.999);
		    call symput ('L' , trim(left(L))); 
		run;

		%let j_dep=1;

		%do k=1 %to 10;
		  %let j=%eval(1+((&k-1)*&L));
		  %let kL=%eval(&k*&L);

		data _NULL_;
		if (&b - &kL) >0  then do;   
		                         k=&k;
		                         j=1+((&k-1)*&L);
		                         kL=&k*&L;
		                       end;
		                  else do; 
		                         k=10; 
		                         j=&j_dep;
		                         kL=&b;
		                       end;

		call symput ('k' , trim(left(k)));
		call symput ('j' , trim(left(j)));
		call symput ('kL' , trim(left(kL)));
		run;

		data poids (keep = bsw&j-bsw&kL &yvar &xvar &classes);  
		  set bs_data;
		run;

		  %do i=&j %to &kL;

		   %put Regression linaire &i termine;

		   %let j_dep=%eval(&kl+1);

	ods listing close;

		proc reg data=poids;
		     model &yvar=&xvar;
		     weight bsw&i;
		&by  by &classes;
		ods output parameterestimates=regout;
		   run;

		proc summary data=regout;
		&by  by &classes;
				var df;
				output out = regoutdf sum = sumdf;
			run;

		data regout;
			set regout;
			ind=1;
		run;
		data regoutdf;
			set regoutdf;
			ind=1;
		run;

		data regout2;
			merge regoutdf regout;
		    by &classes ind; 
			if sumdf < _freq_ or sumdf=. then estimate=.;
			best&i=estimate;
		run;

		   %if (&i =1) %then %do;

		     data bsbeta;
		       set regout2 (keep=&classes best&i variable);
		     run;

		   %end;
		   %else %do;

			 data bsbeta;
			    merge bsbeta regout2 (keep=&classes best&i variable);
			 &by    by &classes;
			 run;

		   %end;
	
		ods listing;

		  %end;
		%end;

		OPTIONS notes;

*** Identification des rpliques non-vides (les "best" sont recods  manquant)***;

		proc means data=bs_data noprint;
		    var bsw1-bsw&b;
		&by  by &classes;
		    output out=bsbeta_nul
		         sum= ybs_nul1-ybs_nul&b;
		run;

		data bsbeta (drop=ybs_nul1-ybs_nul&b);
			merge bsbeta bsbeta_nul;
		&by  by &classes;
			array best{&b};
			array ybs_nul{&b};
			do i = 1 to &b;
				if ybs_nul{i}=0 then best{i}=.;
			end;
		run;

*** Variables pour le niveau de confiance ;

		%let niveau=probit(1-(&alpha/2)); 
		data _NULL_;
			call symput ('ic' ,(1-&alpha)*100);
		run;
		data _NULL_;
	 		call symput ('ic_inf',compress("ic_i&ic"));
	 		call symput ('ic_sup',compress("ic_s&ic"));
		run;

*** Calcul des rsultats ***;

		data est;
		  merge origest bsbeta;
		  &by 	by &classes;
		  rename beta1=Beta;
		  	*** Calcul du nombre de rpliques non-vides ***;
			rep_mod=0;
		  	array best{&b};
			do i=1 to &b;
			if best{i} ne . then rep_mod=rep_mod+1; 
		    end; 
		  if rep_mod ne 0 then bs_var=&r*((rep_mod-1)*(var(of best1-best&b)))/rep_mod;
		  bs_et=sqrt(bs_var);
		  if bs_et ne 0 then t=beta1/bs_et;
		  valeurp=2*(1-probnorm(abs(t)));  * La distribution normale est utilise comme approximation de la distribution t;
		  dep="&yvar";
		  mod=&model_reg;
		  Modele=compress(mod)||': Variable dpendante = '|| compress(dep) ; 
		  drop best1-best&b dep _type_ _freq_ i;
		  if rep_mod ne &b then call symput('flag_reg',1);
		run;

		data bs_reg;
		    set bs_reg est;
		run;

*** Cration du fichier de rsultats ***;

		%let printreg=1;

		proc datasets library=work;
		    delete regout regout2 regoutdf bsbeta bsbeta_nul origest est orig poids;
		run;


%mend regress;



************************************************
****                                        ****
****         RGRESSION LOGISTIQUE          ****
****                                        ****
************************************************;


%macro logreg(yvar,xvar);

		%let model_log=&model_log+1;
		OPTIONS nonotes;

*** Estimations ponctuelles ***;

		proc logistic data=bs_data outest=orig(keep=&classes &inter &xvar) descending noprint;
		  model &yvar=&xvar;
		  &by by &classes;
		  weight &fwgt;
		run;

		proc transpose data=orig out=origest prefix=beta name=param;
		  var &inter &xvar;
		  &by by &classes;
		run;

*** Calcul de la variance bootstrap ***;

		data _NULL_; 
		    L=int((&b/10)+0.999);
		    call symput ('L' , trim(left(L))); 
		run;

		%let j_dep=1;

		%do k=1 %to 10;
		  %let j=%eval(1+((&k-1)*&L));
		  %let kL=%eval(&k*&L);

		data _NULL_;
		if (&b - &kL) >0  then do; 
		                          k=&k;
		                          j=1+((&k-1)*&L);
		                          kL=&k*&L;
		                       end;
		                  else do; 
		                          k=10; 
		                          j=&j_dep;
		                          kL=&b;
		                       end;

		call symput ('k' , trim(left(k)));
		call symput ('j' , trim(left(j)));
		call symput ('kL' , trim(left(kL)));
		run;


		data poids (keep = bsw&j-bsw&kL &yvar &xvar &classes);  
		  set bs_data;
		run;

		  %do i=&j %to &kL;

		  %put Regression logistique &i termine;
		 
		    %let j_dep=%eval(&kl+1);

	ods listing close;

			proc logistic data=poids descending;
		      model &yvar=&xvar;
		      &by by &classes;
		      weight bsw&i;
			ods output parameterestimates=regout;
		    run;

		proc summary data=regout;
		&by  by &classes;
				var df;
				output out = regoutdf sum = sumdf;
			run;

		data regout;
			set regout;
			ind=1;
		run;
		data regoutdf;
			set regoutdf;
			ind=1;
		run;

		data regout2;
			merge regoutdf regout;
		    by &classes ind; 
			if sumdf < _freq_ or sumdf=. then estimate=.;
			best&i=estimate;
		run;

		    %if (&i =1) %then %do;

		     data bsbeta;
		       set regout2 (keep=&classes best&i variable);
		     run;

		   %end;
		   %else %do;

			 data bsbeta;
			    merge bsbeta regout2 (keep=&classes best&i variable);
			 &by    by &classes;
			 run;

		   %end;
	
		ods listing;

		  %end;
		%end;

		OPTIONS notes;

*** Identification des rpliques non-vides (les "best" sont recods  manquant)***;

		proc means data=bs_data noprint;
		    var bsw1-bsw&b;
		&by  by &classes;
		    output out=bsbeta_nul
		         sum= ybs_nul1-ybs_nul&b;
		run;

		data bsbeta (drop=ybs_nul1-ybs_nul&b);
			merge bsbeta bsbeta_nul;
		&by  by &classes;
			array best{&b};
			array ybs_nul{&b};
			do i = 1 to &b;
				if ybs_nul{i}=0 then best{i}=.;
			end;
		run;

*** Variables pour le niveau de confiance ;

		%let niveau=probit(1-(&alpha/2)); 
		data _NULL_;
			call symput ('ic' ,(1-&alpha)*100);
		run;
		data _NULL_;
	 		call symput ('ic_inf',compress("ic_i&ic"));
	 		call symput ('ic_sup',compress("ic_s&ic"));
		run;

*** Calcul des rsultats ***;

		data est;
		  merge origest bsbeta;
		  &by 	by &classes;
		  rename beta1=Beta;
		  	*** Calcul du nombre de rpliques non-vides ***;
			rep_mod=0;
		  	array best{&b};
			do i=1 to &b;
			if best{i} ne . then rep_mod=rep_mod+1; 
			end; 
		  if rep_mod ne 0 then bs_var=&r*((rep_mod-1)*(var(of best1-best&b)))/rep_mod;
		  bs_et=sqrt(bs_var);
		  if bs_et ne 0 then Wald=(beta1/bs_et)*(beta1/bs_et);
		  valeurp=1-probchi(Wald,1);
		  cotes=exp(beta1);
		  lo=beta1-&niveau*bs_et;
		  hi=beta1+&niveau*bs_et;
		  &ic_inf=exp(lo);
		  &ic_sup=exp(hi);
		  dep="&yvar";
		  mod=&model_log;
		  Modele=compress(mod)||': Variable dpendante = '|| compress(dep) ; 
		  drop best1-best&b dep _type_ _freq_ i;
		  if rep_mod ne &b then call symput('flag_log',1);
		run;

		data bs_reglg; 
		     set bs_reglg est;
		run;

*** Cration du fichier de rsultats ***;

		%let printlog=1;

		proc datasets library=work;
		    delete regout regout2 regoutdf bsbeta bsbeta_nul origest est orig poids;
		run;


%mend logreg;



************************************************
****                                        ****
****                 CHI-2                  ****
****                                        ****
************************************************;

**** Note: La mthode de calcul est la mme que celle utilise par Wesvar;


*** Pour permettre aux utilisateurs d'avoir des formats avec les donnes; ***;

		proc format;
			value toblankexisting;
		run;


%macro chi2(var1,var2);

%let printchi2=1;
%let test_chi2=&test_chi2+1;

	%local nvar class_list temp;

	%if "&classes" ne " " %then %do;
		%let temp = %sysfunc(compbl(&classes));
		*** Insrer une virgule entre les variables de la liste CLASSES   ***;
		%let class_list = %sysfunc(translate(&temp, ",", " "));

	    ***	Sparer la liste CLASSES en plusieurs variables ***;
		%let nvar = 1;
		%let v = %scan(&classes,&nvar);
		%do %while (%length(&v) > 0);
			data _NULL_;
				call symput('clas'!!left(&nvar),"&v");
			run;

			%let nvar = %eval(&nvar+1);
			%let v = %scan(&classes,&nvar);
		%end; /* while */
		%let nvar = %eval(&nvar-1);
	%end; /* if classes */

***	Driver les poids normaliss pour obtenir le test de PEARSON appropri ***;

	proc sql;
		create table nwgt as
		select %if "&classes" ne " " %then &class_list,; 
				&var1, &var2, &fwgt/mean(&fwgt) as normwgt
		from bs_data
		%if "&classes" ne " " %then
		group by &class_list;
		;
	quit;

***	Driver les estimations en utilisant les poids finaux normaliss  ***;

	proc freq data = nwgt noprint;
	&by by &classes;
		table &var1 * &var2/chisq outpct out = temp0;
		weight normwgt;
		output out = chi2test pchi;
	run;

***	Lire les rsultats pour mettre un avertissement en prsence de cellules avec 5 observations ou moins ***;

	proc sql;
		create table chi2test as
		select distinct c.*, case 
						when t.count <= 5 then '(*)'
						else '   ' end as warn_msg
		from chi2test as c, temp0 as t
		%if "&classes" ne " " %then %do;
		where t.percent ne . 
			%do i = 1 %to &nvar;
				and c.&&clas&i = t.&&clas&i
			%end;
		%end;
		order by %if "&classes" ne " " %then 
					%do i = 1 %to &nvar;
						c.&&clas&i,
					%end;
				 warn_msg desc
		;
	quit;

	data chi2test;
		set chi2test;
		by &classes;
		%if "&classes" ne " " %then %do;
			if first.&&clas&nvar;	
		%end;
		%else %do;
			if _N_ = 1;
		%end;
	run;


%local lowcell;
%let lowcell = 0;	

	data temp0;
		set temp0;
		if percent;
		/* convert percentages into decimal numbers */
		percent = percent / 100;
		if count le 5 then
			call symput('lowcell',1);
	run;


***	Driver les tableaux pour les rpliques bootstrap en utilisant les poids bootstrap ***;

	%local _rep;

	%do _rep = 1 %to &b;
		*** Diviser en sous-ensembles de 50 poids  la fois pour augmenter efficacit  ***;
		%if %eval((&_rep - 1)/50) = %sysevalf((&_rep-1)/50,ceil) %then %do;
			data __;
				set bs_data (keep = &classes &var1 &var2 bsw&_rep-bsw%sysfunc(min(&_rep+49,&b)));
			run;
		%end;
			proc freq data = __ noprint;
	&by 		by &classes;
				table &var1 * &var2/outpct out = sortie;
				weight bsw&_rep;
			run;

			data sortie;
				set sortie;
				if percent;
				rep = &_rep;
				percent = percent/100;
			run;

		***	Ajouter les rsultats dans REP_TABLES	***;

		%if &_rep = 1 %then %do;
			data rep_tables;
				set sortie;
			run;
		%end;
		%else %do;
			proc append data = sortie base = rep_tables;
			run;
		%end;
	%end;
	
	proc sql noprint;
		drop table sortie
		;
	quit;

***	Compter nombre observations. Sauvegarder dans une variable macro	***;

	%local nobs dsid rc;
		%let dsid = %sysfunc(open(bs_data));
	%let nobs = %sysfunc(attrn(&dsid,nobs));
		%let rc = %sysfunc(close(&dsid));
 
***	Obtenir nombre observations par variable(s) de ventilation (CLASSES) ***;

	%if "&classes" ne " " %then %do;
		proc summary data = bs_data;
			class &classes;
			output out = classification (where = (&clas1
				%do i = 2 %to &nvar;
					and &&clas&i
				%end;	));
		run;
	%end;

***	Calculer les pourcentages des marginales ***;

	proc summary data = temp0;
		class &classes &var1 &var2;
		var percent;
		output out = marg sum = margin;
	run;

	proc sql;
		create table marg0 as
		select distinct t.*, a.margin as pr, b.margin as pc
		from temp0 as t, marg as a, marg as b
		where /* extraire les marginales - ranges (from MARG as A) */
				(t.&var1 = a.&var1 and  a.&var2 = .
				  %if "&classes" ne " " %then
					%do i = 1 %to &nvar;
						and t.&&clas&i = a.&&clas&i
					%end;)
		and /*  extraire les marginales - colonnes (from MARG as B) */
				(t.&var2 = b.&var2 and  b.&var1 = .
				  %if "&classes" ne " " %then
					%do i = 1 %to &nvar;
						and t.&&clas&i = b.&&clas&i
					%end;)
		;
	quit;

*** 	... et rpter pour chaque rpliques bootstrap	***;

	proc summary data = rep_tables;
		class rep &classes &var1 &var2;
		var percent;
		output out = marg sum = margin;
	run;

	proc sql;
		create table marginales as
		select distinct t.*, a.margin as pr, b.margin as pc
		from rep_tables as t, marg as a, marg as b
		where (t.&var1 = a.&var1 and t.rep = a.rep and  a.&var2 = .
				  %if "&classes" ne " " %then
					%do i = 1 %to &nvar;
						and t.&&clas&i = a.&&clas&i
					%end;)
			  and (t.&var2 = b.&var2 and t.rep = b.rep and  b.&var1 = .
				  %if "&classes" ne " " %then
					%do i = 1 %to &nvar;
						and t.&&clas&i = b.&&clas&i
					%end;)
		order by rep, %if "&classes" ne " " %then &class_list,; 
				 &var1, &var2
		;
	quit;

***	Driver les estimations de variance (i.e. Vij, Vi. et V.j	    ***
***							ou	   VARPIJ, VC et VR respectivement)	***;

proc sql;
	****************
	*	CELLULES   *
	****************;
		create table vij0 as
		select distinct r.rep, 
						%if "&classes" ne " " %then
							%do i = 1 %to &nvar;
								r.&&clas&i,
							%end;
						r.&var1, r.&var2, t.percent as p0, 
			 (r.percent - t.percent)**2  as diffs
		from marginales as r, marg0 as t
		where t.&var1 = r.&var1 and t.&var2 = r.&var2
				%if "&classes" ne " " %then
				  	%do i = 1 %to &nvar;
						and t.&&clas&i = r.&&clas&i 
					%end;
		;
		create table vij as
		select distinct %if "&classes" ne " " %then &class_list,; &var1, &var2, p0,
		/* Utiliser approche o les rpliques vides sont ignores et seulement les rpliques non-vides sont utilises	*/
			   &r*(1/count(*))*sum(diffs) as varpij
		from vij0
		group by %if "&classes" ne " " %then &class_list,; &var1, &var2
		;

	****************
	*	RANGES    *
	****************;
		create table vi0 as
		select distinct rep, 
				%if "&classes" ne " " %then
					%do i = 1 %to &nvar;
						r.&&clas&i,
					%end;
			 r.&var1, 
			 t.pr as pr0,
			 (r.pr - t.pr)**2  as diffs_r
		from marginales as r, marg0 as t
		where t.&var1 = r.&var1 
				%if "&classes" ne " " %then
				  	%do i = 1 %to &nvar;
						and t.&&clas&i = r.&&clas&i 
					%end;
		;
		create table vi as
		select distinct %if "&classes" ne " " %then &class_list,; &var1, pr0,
		/* Utiliser approche o les rpliques vides sont ignores et seulement les rpliques non-vides sont utilises	*/
			   &r*(1/count(*))*sum(diffs_r) as vr
		from vi0
		group by %if "&classes" ne " " %then &class_list,; &var1
		;

	*****************
	*	COLONNES    *
	*****************;
		create table vj0 as
		select distinct rep, 
				%if "&classes" ne " " %then
					%do i = 1 %to &nvar;
						r.&&clas&i,
					%end;
			 r.&var2, 
			 t.pc as pc0,
			 (r.pc - t.pc)**2  as diffs_c
		from marginales as r, marg0 as t
		where t.&var2 = r.&var2
				%if "&classes" ne " " %then
				  	%do i = 1 %to &nvar;
						and t.&&clas&i = r.&&clas&i 
					%end;
		;
		create table vj as
		select distinct %if "&classes" ne " " %then &class_list,; &var2, pc0,
		/* Utiliser approche o les rpliques vides sont ignores et seulement les rpliques non-vides sont utilises	*/
			   &r*(1/count(*))*sum(diffs_c) as vc
		from vj0
		group by %if "&classes" ne " " %then &class_list,; &var2
		;

*** Regrouper toutes les variances (cellules et marginales) dans le tableau VARPIJ ***;

	create table varpij as 
	select distinct %if "&classes" ne " " %then
						%do i = 1 %to &nvar;
							r.&&clas&i,
						%end;
		   r.&var1, r.&var2, p0, pr0, pc0,
		   varpij, vr, vc
	from vi as i, vj as j, vij as r
	where i.&var1 = r.&var1 and j.&var2 = r.&var2
		  %if "&classes" ne " " %then 
			%do i = 1 %to &nvar;
				and i.&&clas&i = r.&&clas&i 
				and j.&&clas&i = r.&&clas&i 
				and i.&&clas&i = j.&&clas&i
			%end;
	;

********************************************************************
*	   RS3 (CHI-2 avec ajustement Rao-Scott du second degr)       *
********************************************************************;

***	Prparer entre pour RS3 ***;

	*** Calculer les ratios Vi./Pi.   ***;
		create table vipi as
		select distinct %if "&classes" ne " " %then &class_list,; &var1, vr/pr0 as ratr
		from varpij
		order by %if "&classes" ne " " %then &class_list,; &var1
		;

	*** Calculer les ratios V.j/P.j  ***;
		create table vjpj as
		select distinct %if "&classes" ne " " %then &class_list,; &var2, vc/pc0 as ratc
		from varpij
		order by %if "&classes" ne " " %then &class_list,; &var2
		;
		quit;

***	Pour certaines raisons, il arrive que certaines ranges sont dupliques.  Les 2 tapes suivantes les liminent...	***;

	data vipi;
		set vipi;
		by &classes &var1;
		if first.&var1;
	run;

	data vjpj;
		set vjpj;
		by &classes &var2;
		if first.&var2;
	run;

	proc sql;

	*** Somme sur les ranges de Vi./Pi. ***;
		create table vipi2 as
		select distinct %if "&classes" ne " " %then &class_list,; 
				sum(ratr) as sum_vi_pi
		from vipi
		%if "&classes" ne " " %then group by &class_list;
		;

	*** Somme sur les colonnes de V.j/P.j ***;
		create table vjpj2 as
		select distinct %if "&classes" ne " " %then &class_list,; 
				sum(ratc) as sum_vj_pj
		from vjpj
		%if "&classes" ne " " %then group by &class_list;
		;

	*** Somme sur les ranges et colonnes de Vij/(Pi.*P.j) ***;
		create table vijpipj as
		select distinct %if "&classes" ne " " %then &class_list,; 
				sum(varpij/(pr0*pc0)) as sum_vij_pipj
		from varpij
		%if "&classes" ne " " %then group by &class_list;
		;

	*** Regrouper les somme prcdentes ***;
		create table vps as
		select v.*, i.sum_vi_pi, j.sum_vj_pj, ij.sum_vij_pipj
		from varpij as v, vipi2 as i, vjpj2 as j, vijpipj as ij
		%if "&classes" ne " " %then %do;
		where v.&clas1 = i.&clas1 and v.&clas1 = j.&clas1 and v.&clas1 = ij.&clas1 
			%do i = 2 %to &nvar;
				and v.&&clas&i = i.&&clas&i and v.&&clas&i = j.&&clas&i and v.&&clas&i = ij.&&clas&i 
			%end;
		%end;
		;
	quit;


***  Un tableau de toutes les combinaisons du numro de rplique, de &CLASSES, VAR1  ***
***  et VAR2 est requis pour tenir compte des cas o certaines rpliques ont des     ***
***  cellules vides et qui sont reprsentes dans le tableau crois initial (i.e.    ***
***  avec les poids finaux).  REPS contient une occurence de chaque numro de        ***
***  rplique qui sera utilis pour faire un produit cartsien avec le tableau crois***
***  de &CLASSES, VAR1 et VAR2.				 					   				     ***;

	data reps;
		do rep = 1 to &b;
			output;
		end;
	run;

***	Nombre observation par &CLASSES (_FREQ_) est ncessaire pour calculer le 	***
***	vecteur Y, o tous les lments sont multiplis par SQRT(n). [n <- _FREQ_]  ***;

	%if "&classes" ne " " %then %do;
		proc sql;
			create table vps as
			select v.*, _FREQ_
			from vps as v, classification as c
			where v.&clas1 = c.&clas1
				  %do i = 2 %to &nvar;
				  	and v.&&clas&i = c.&&clas&i
				  %end;
			;
			create table rep_tables as
			select r.*, f._FREQ_
			from rep_tables as r, classification as f
			where r.&clas1 = f.&clas1
				  %do i = 2 %to &nvar;
				  	and r.&&clas&i = f.&&clas&i
				  %end;
			;

	***	Crer le tableau mentionn ci-dessus	***;

		create table lut as
		select distinct rep, 
						%do i = 1 %to &nvar;
							a.&&clas&i,
						%end; 
						&var1, &var2, _freq_
		from reps, 
			 (select distinct &class_list, &var1 from marg0) as a,
			 (select distinct &class_list, &var2 from marg0) as b,
			 classification as f
		where a.&clas1 = f.&clas1 and b.&clas1 = f.&clas1 and a.&clas1 = b.&clas1
			  %do i = 2 %to &nvar;
			  	and a.&&clas&i = f.&&clas&i and b.&&clas&i = f.&&clas&i and a.&&clas&i = b.&&clas&i
			  %end;
		;
	%end;
	%else %do;
		data vps;
			set vps;
			_FREQ_ = &nobs;
		run;
		data rep_tables;
			set rep_tables;
			_FREQ_ = &nobs;
		run;

		proc sql;

	***	Crer le tableau mentionn ci-dessus si aucune variable &CLASSES ***;

			create table lut as
			select distinct rep, &var1, &var2, &nobs as _FREQ_
			from reps, marg0
			;
	%end;


*** REP_TABLE peut avoir certaines ranges non-incluses.  Ceci se produit quand ***
*** certaines rpliques ont des cellules vides et qui sont reprsentes dans le ***
*** tableau crois initial. Cette tape gnre des lignes o PERCENT est        ***
*** manquant.  Elles sont changes  0 aprs.                                   ***;

	create table rep_tables2 as
	select l.*, count, percent
	from lut as l left join rep_tables as a 
	on l.rep = a.rep and
	   l.&var1 = a.&var1 and l.&var2 = a.&var2
	   %if "&classes" ne " " %then 
		%do i = 1 %to &nvar;
			and l.&&clas&i = a.&&clas&i
		%end;
	order by l.rep, 
		  %if "&classes" ne " " %then
	   		%do i = 1 %to &nvar;
				&&clas&i,
			%end; 
			l.&var1, l.&var2
	;
	update rep_tables2
	set percent = 0, count = 0
	where percent = .
	;

	***	Calculer les Ys pour les estims (YS) et pour les rpliques (YS_REP)	***;

	create table lut_r as
	select distinct %if "&classes" ne " " %then 
						%do i = 1 %to &nvar;
							l.&&clas&i,
						%end;
		   l.&var1, pr0, l._freq_
	from lut as l left join vps as v
	on l.&var1 = v.&var1
		%if "&classes" ne " " %then 
			%do i = 1 %to &nvar;
				and l.&&clas&i = v.&&clas&i 
			%end;
	;
	create table lut_c as
	select distinct %if "&classes" ne " " %then 
						%do i = 1 %to &nvar;
							l.&&clas&i,
						%end;
		   l.&var2, pc0
	from lut as l left join vps as v
	on l.&var2 = v.&var2
		%if "&classes" ne " " %then 
			%do i = 1 %to &nvar;
				and l.&&clas&i = v.&&clas&i 
			%end;

	;
	create table lut_rc as
	select %if "&classes" ne " " %then
			 %do i = 1 %to &nvar;
				r.&&clas&i,
			 %end;
		   r.&var1, c.&var2, pr0, pc0, _freq_
	from lut_c as c, lut_r as r
	%if "&classes" ne " " %then %do;
	where r.&clas1 = c.&clas1
			%do i = 1 %to &nvar;
				and r.&&clas&i = c.&&clas&i
			%end;
	%end;
	;
	create table ys as
	select %if "&classes" ne " " %then
			 %do i = 1 %to &nvar;
				l.&&clas&i,
			 %end;
		   l.&var1, l.&var2, p0, l.pr0, l.pc0, l._freq_
	from lut_rc as l left join vps as v
	on l.&var1 = v.&var1 and l.&var2 = v.&var2 
		%if "&classes" ne " " %then 
			%do i = 1 %to &nvar;
				and l.&&clas&i = v.&&clas&i
			%end;
	order by l.&var1, l.&var2
	;
	quit;

	data ys;
		set ys;
		if p0 = . then
			p0 = 0;
		y = sqrt(_FREQ_)*(p0 - (pr0*pc0));
		p = pr0*pc0;
	run;

	proc sql;
		create table ys_rep as
		select *, sum(percent) as pr
		from rep_tables2
		/*	Par VAR1 pour avoir les proportions par ranges	*/
		group by rep, %if "&classes" ne " " %then &class_list,; &var1
		;
		create table ys_rep as
		select *, sum(percent) as pc
		from ys_rep
		/*	Par VAR2 pour avoir les proportions par colonnes	*/
		group by rep, %if "&classes" ne " " %then &class_list,; &var2
		order by rep, %if "&classes" ne " " %then &class_list,; &var1, &var2
		;
		create table ys_rep as
		select *, sqrt(_FREQ_)*(percent - (pr*pc)) as yg
		from ys_rep
		order by rep, %if "&classes" ne " " %then &class_list,; &var1, &var2
		;
		/* Gnrer les (Yig-Yi) pour chaque rplique bootstrap	*/
		create table yig_yi as
		select r.*, t.p, yg - y as yig_yi
		from ys_rep as r left join ys as t
		on r.&var1 = t.&var1 and r.&var2 = t.&var2 
			%if "&classes" ne " " %then 
			%do i = 1 %to &nvar;
				and r.&&clas&i = t.&&clas&i 
			%end;
		order by r.rep, %if "&classes" ne " " %then &class_list,; r.&var1, r.&var2
		;
	quit;

	data yig_yi;
		set yig_yi;
		by rep &classes &var1 &var2;
		%if "&classes" ne " " %then %do;
			if first.&&clas&nvar then i = 0;
		%end;
		%else %do;
			if first.rep then i = 0;
		%end;
		/* Crer un index pour la prochaine tape */
		i + 1;
	run;


	proc sql;
	***	Lire le tableau YIG_YI deux fois, une fois en tant que (Yig-Yi) et la deuxime fois en ***
	***	tant que (Yjg-Yj) pour driver les Bij.     										   ***;
		create table bij as
		select distinct %if "&classes" ne " " %then 
							%do i = 1 %to &nvar;
								i.&&clas&i,  
							%end;
				i.i as i, 
				j.i as j, 
				i.p as pi, 
				j.p as pj, 
				(1/&b)*sum(i.yig_yi * j.yig_yi) as Bij
		from yig_yi as i, yig_yi as j
		where i.rep = j.rep 
			%if "&classes" ne " " %then 
				%do i = 1 %to &nvar;
					and i.&&clas&i = j.&&clas&i 
				%end;
		group by %if "&classes" ne " " %then 
				%do i = 1 %to &nvar;
					i.&&clas&i,  
				%end;
				i.&var1, i.&var2, j.&var1, j.&var2
		;

***	Crer les rsultats RS3 ***;

	%if "&classes" ne " " %then %do;
		create table rs3 as
		select distinct %do i = 1 %to &nvar;
							a.&&clas&i,
						%end;
				c.warn_msg, 
				b.bii_pi,	/* Numrateur de la formule de degr de libert (nu) (pas encore au carr) */
				c._PCHI_, 
				sum((Bij**2)/(pi*pj)) as BijPiPj	/* Dnominateur de la formule de degr de libert (nu) */
		from bij as a, 
				(/* Extraire Bii/Pi, i.e. Bij's o i = j */
				 select distinct &class_list, sum(Bij/pi) as bii_pi 
				 from bij 
				 where i=j 
				 group by &class_list
				) as b,
			 chi2test as c
		where a.&clas1 = b.&clas1 and a.&clas1 = c.&clas1
			  %do i = 2 %to &nvar;
				and a.&&clas&i = b.&&clas&i and a.&&clas&i = c.&&clas&i
			  %end;
		group by a.&clas1
				 %do i = 2 %to &nvar;
					, a.&&clas&i
				 %end;
		;
	%end;
	%else %do;
		create table rs3 as
		select distinct c.warn_msg, 
			   b.bii_pi, 	/* Numrateur de la formule de degr de libert (nu) (pas encore au carr) */
			   c._PCHI_, 
			   sum((Bij**2)/(pi*pj)) as BijPiPj	/* Dnominateur de la formule de degr de libert (nu) */
		from bij as a,
			(/* Extraire Bii/Pi, i.e. Bij's o i = j */
			 select distinct sum(Bij/pi) as bii_pi 
			 from bij 
			 where i=j
			) as b,
			 chi2test as c
		;
	%end;
	quit;


************************************************************
*		Prparer pour impression des rsultats du CHI-2	   *
************************************************************;

	proc sql;
		create table chi2 as
		select distinct %if "&classes" ne " " %then &class_list,;
			   "&var1" as var1, "&var2" as var2,
			   "Chi-2 ajust (ajust. R.-S. du sec. degr)"!!warn_msg as statistic, 
			   (bii_pi**2)/bijpipj as Deg_Freedom,
			   ((bii_pi**2)/bijpipj)*_PCHI_/bii_pi as chi2_value,
			   1 - probchi((bii_pi**2)/bijpipj*_PCHI_/bii_pi, (bii_pi**2)/bijpipj) as p_value
		from rs3
		;
	quit;

	data chi2;
		set chi2;
		test_num=&test_chi2;
		Test = compress(test_num)||':   Variables:  '|| compress(var1) ||'  VS  '||compress(var2); 
	run;

	data allchi2;
		set allchi2 chi2;
	run;


***	Crer un tableau crois contenant les statistiques requises	***;

	proc sql noprint;
		select type
		into :type1
		from sashelp.vcolumn
		where libname = "WORK" and memname = "VARPIJ" and upcase(name) = "%upcase(&var1)"
		;
		select type
		into :type2
		from sashelp.vcolumn
		where libname = "WORK" and memname = "VARPIJ" and upcase(name) = "%upcase(&var2)"
		;
	quit;

	proc sql;
		create table cross_in as
		select %if "&classes" ne " " %then &class_list,;
			   &var1, &var2, 
			   p0 label = "Pourcentage",
			   varpij label = "Variance",
			   sqrt(varpij)/p0 as cv label = "_C.V._"
		from varpij
		order by %if "&classes" ne " " %then &class_list,;&var1, &var2
		;

		create table cross_row as
		select distinct %if "&classes" ne " " %then &class_list,; &var1,
			   pr0 label = "Pourcentage",
			   vr label = "Variance",
			   sqrt(vr)/pr0 as cv label = "_C.V._"
		from varpij
		order by %if "&classes" ne " " %then &class_list,; &var1
		;

		create table cross_col as
		select distinct %if "&classes" ne " " %then &class_list,;
			   %if &type1 = num %then .; %else " "; as &var1, &var2, 
			   pc0 label = "Pourcentage",
			   vc label = "Variance",
			   sqrt(vc)/pc0 as cv label = "_C.V._"
		from varpij
		order by %if "&classes" ne " " %then &class_list,; &var1, &var2
		;
	quit;

	proc transpose data = cross_row out=row_trans(drop=_name_ rename = (col1 = total));
		by &classes &var1;
		var pr0 vr cv;
	run;

	proc transpose data=cross_col out = col_trans(drop=_name_);
		by &classes &var1;
		id &var2;
		format &var2;
	run;

	proc transpose data = cross_in out = in_trans (drop = _name_);
		by &classes &var1;
		id &var2;
		format &var2;
	run;

	%if &type2 = num %then %do;
		proc sql noprint;
			select count(distinct &var2)
			into :ncolumns
			from temp0
			;
		quit;
		data _null_;
			call symput('colx',compress('val'!!&ncolumns));
		run;
		proc sql noprint;
			select distinct &var2 format=toblankexisting.
			into :val1-:&colx
			from temp0
			;
		quit;

		data col_trans;
			set col_trans;
			%do i = 1 %to &ncolumns;
				%let valeur = _&&val&i;
				rename &valeur = &var2&valeur;
				format &valeur 11.4;
			%end;
		run;

		data in_trans;
			set in_trans;
			%do i = 1 %to &ncolumns;
				%let valeur = _&&val&i;
				rename &valeur = &var2&valeur;
				format &valeur 11.4;
			%end;
		run;
	%end;

	data col_trans;
		set col_trans;
		total=.;
		var1="&var1";
		var2="&var2";
	run;

	data temp;
		var1="&var1";
		var2="&var2";
	run;

	proc sql;
		create table crosstab as
		select a.*, b.total, c.*
		from in_trans as a, row_trans as b, temp as c
		where %if "&classes" ne " " %then 
				%do i = 1 %to &nvar;
					a.&&clas&i = b.&&clas&i and
				%end;		
			  a.&var1 = b.&var1 and
			  a._label_ = b._label_
		 union 
		select *
		from col_trans
		;
	quit;

	data crosstab (drop =  test_num var1 var2) ;
		set crosstab;
		test_num=&test_chi2;
		call symput ('test_num2' ,compress(&test_chi2));
		Test = compress(test_num)||':   Variables:  '|| compress(var1) ||'  VS  '||compress(var2); 
	run;

	data crosstab&test_num2;
		set crosstab;
	run;

	proc datasets library=work;
	      delete chi2 ;
	run;
	quit;
 
	%if "&classes" ne " " %then %do;
	proc sql;
			drop table bij, chi2test, classification, lut, lut_r, lut_c, lut_rc, marg, marg0, 
				   marginales, nwgt, reps, rep_tables, rep_tables2, rs3, temp0,  
				   varpij, vijpipj, vipi, vipi2, 
				   vjpj, vjpj2, vi, vi0, vij, vij0, vj, vj0, vps, yig_yi,ys, ys_rep, __,
                   crosstab, cross_col, cross_in, cross_row, in_trans, row_trans, col_trans, temp;
	quit;
	%end;
	%else %do;
	proc sql;
			drop table bij, chi2test, lut, lut_r, lut_c, lut_rc, marg, marg0, 
				   marginales, nwgt, reps, rep_tables, rep_tables2, rs3, temp0,  
				   varpij, vijpipj, vipi, vipi2, 
				   vjpj, vjpj2, vi, vi0, vij, vij0, vj, vj0, vps, yig_yi,ys, ys_rep, __,
                   crosstab, cross_col, cross_in, cross_row, in_trans, row_trans, col_trans, temp;
	quit;
	%end;

	title;

%mend chi2;  *** Fin de la macro CHI2 ***;



*********************************************************************************;
*** Section 5: DCLARATION DES MACROS POUR IMPRESSION DES RSULTATS           ***;
***       Voir la documentation du programme pour une description             ***;
*********************************************************************************;


****************;

%macro prnttot;

****************;

%if &printtot=1 %then %do;

*** Prparation du fichier de rsultats en prsence de variable(s) de ventilation***;

		&by  data alltots;
		&by  	set alltots (drop = &classes);
		&by     ind1=3; &by  ind2=3; &by  ind3=3; &by  ind4=3;&by   ind5=3; &by  ind6=3; &by  ind7=3;&by   ind8=3;&by   ind9=3;&by   ind10=3;  
		&by     %do i=1 %to &number;
		&by  	     if cla&i=" " then cla&i="Tous";
		&by  	     if cla&i="Tous" then ind&i=1;  &by  else ind&i=2;
		&by     &&cla&i=cla&i;
		&by  	%end;
		&by  	run;

		&by   proc sort data=alltots;
		&by     by ind1 ind2 ind3 ind4 ind5 ind6 ind7 ind8 ind9 ind10  &classes;
		&by   run;

*** Imprime les rsultats de la macro total ***;

		proc print data=alltots label noobs;
		   title ;
		   title2 ;
		   title3 ;
		   title4 "Estimation de la variance  l'aide du bootstrap pour des TOTAUX";
		   title5 "(&B rpliques bootstrap)";
		   title6 ;
		   %if (&deff=1) %then %do;
		   var &classes var n Estimation bs_et bs_cv &ic_inf &ic_sup deff;
		   format Estimation bs_et &ic_inf &ic_sup deff 11.2;
		   label  bs_et = "Erreur-type"
		          Estimation = "Total"
				  bs_cv = "Coeff. de variation"
				  var = "Variable"
				  n = "Taille d'chantillon" 
				  &ic_inf = "Borne inf. intervalle de confiance &ic%"
				  &ic_sup = "Borne sup. intervalle de confiance &ic%"
				  deff = "Effet de plan";
		   %end;
		   %else %do;
		   var &classes var n Estimation bs_et bs_cv &ic_inf &ic_sup;
		   format Estimation bs_et &ic_inf &ic_sup 11.2;
		   label  bs_et = "Erreur-type"
		          Estimation = "Total"
				  bs_cv = "Coeff. de variation"
				  var = "Variable"
				  n = "Taille d'chantillon" 
				  &ic_inf = "Borne inf. intervalle de confiance &ic%"
				  &ic_sup = "Borne sup. intervalle de confiance &ic%";
		   %end;
		run;
%end;

%mend prnttot;


****************;

%macro prntrat;

****************;

%if &printrat=1 %then %do;

*** Prparation du fichier de rsultats en prsence de variable(s) de ventilation***;

		&by  data allrats;
		&by  	set allrats(drop = &classes);
		&by     ind1=3; &by  ind2=3; &by  ind3=3; &by  ind4=3;&by   ind5=3; &by  ind6=3; &by  ind7=3;&by   ind8=3;&by   ind9=3;&by   ind10=3;  
		&by     %do i=1 %to &number;
		&by  	     if cla&i=" " then cla&i="Tous";
		&by  	     if cla&i="Tous" then ind&i=1;  &by  else ind&i=2;
		&by     &&cla&i=cla&i;
		&by  	%end;
		&by  	run;

		&by   proc sort data=allrats;
		&by     by ind1 ind2 ind3 ind4 ind5 ind6 ind7 ind8 ind9 ind10  &classes;
		&by   run;

*** Imprime les rsultats de la macro ratio ***;

		proc print data=allrats label noobs;
		   title ;
		   title2 ;
		   title3 ;
		   title4 "Estimation de la variance  l'aide du bootstrap pour des RATIOS";
		   title5 "(&B rpliques bootstrap)";
		   title6 ;
		     label bs_et = "Erreur-type"
		          Estimation = "Ratio"
				  bs_cv = "Coeff. de variation"
				  var1 = "Numrateur"
				  var2 = "Dnominateur"
				  n1 = "Taille numrateur" 
				  &ic_inf = "Borne inf. intervalle de confiance &ic%"
				  &ic_sup = "Borne sup. intervalle de confiance &ic%"
                  rep_mod = "Nombre de rpliques utilises";
		   %if (&deff=1 and &flag_rat=0) %then %do;
			 var &classes var1 var2 n1 Estimation bs_et bs_cv &ic_inf &ic_sup deff;
		     format  bs_et &ic_inf &ic_sup Estimation 11.4 deff 11.2;
			 label deff = "Effet de plan";
		   %end;
		   %else %if (&deff=0 and &flag_rat=0) %then %do;
			 var &classes var1 var2 n1 Estimation bs_et bs_cv &ic_inf &ic_sup;
		     format  bs_et &ic_inf &ic_sup Estimation 11.4;
		   %end;
		   %else %if (&deff=1 and &flag_rat=1) %then %do;
			 var &classes var1 var2 n1 Estimation bs_et bs_cv &ic_inf &ic_sup deff rep_mod;
		     format  bs_et &ic_inf &ic_sup Estimation 11.4 deff 11.2;
			 label deff = "Effet de plan";
			 	footnote1 "----------------------------------------------------------------------------------------------------------";
				footnote2 "  Attention: Certaines rpliques n'ont pas t utilises pour le calcul de la variance. Ces    ";
                footnote3 "             rpliques ne contiennent aucune observation ayant la caractristique du dnominateur.";
				footnote4 "             Se rfrer  la colonne 'Nombre de rpliques utilises' pour connatre le nombre utilis.";
				footnote5 "----------------------------------------------------------------------------------------------------------";
		   %end;
		   %else %if (&deff=0 and &flag_rat=1) %then %do;
			 var &classes var1 var2 n1 Estimation bs_et bs_cv &ic_inf &ic_sup rep_mod;
		     format  bs_et &ic_inf &ic_sup Estimation 11.4;
			 	footnote1 "---------------------------------------------------------------------------------------------------------";
				footnote2 "  Attention: Certaines rpliques n'ont pas t utilises pour le calcul de la variance. Ces    ";
                footnote3 "             rpliques ne contiennent aucune observation ayant la caractristique du dnominateur.";
				footnote4 "             Se rfrer  la colonne 'Nombre de rpliques utilises' pour connatre le nombre utilis.";
				footnote5 "---------------------------------------------------------------------------------------------------------";
		   %end;
		run;
%end;
footnote;

%mend prntrat;

****************;

%macro prntdiff;

****************;

%if &printdif=1 %then %do;

*** Prparation du fichier de rsultats en prsence de variable(s) de ventilation***;

		&by  data diffrat;
		&by  	set diffrat(drop = &classes);
		&by     ind1=3; &by  ind2=3; &by  ind3=3; &by  ind4=3;&by   ind5=3; &by  ind6=3; &by  ind7=3;&by   ind8=3;&by   ind9=3;&by   ind10=3;  
		&by     %do i=1 %to &number;
		&by  	     if cla&i=" " then cla&i="Tous";
		&by  	     if cla&i="Tous" then ind&i=1;  &by  else ind&i=2;
		&by     &&cla&i=cla&i;
		&by  	%end;
		&by  	run;

		&by   proc sort data=diffrat;
		&by     by ind1 ind2 ind3 ind4 ind5 ind6 ind7 ind8 ind9 ind10  &classes;
		&by   run;


*** Imprime les rsultats de la macro diff_rat ***;

	   proc print data=diffrat label noobs width=minimum;
		   title ;
		   title2 ;
		   title3 ;
		   title4 "Estimation de la variance  l'aide du bootstrap pour des";
		   title5 "DIFFRENCES ENTRE RATIOS";
		   title6 "(&B rpliques bootstrap)";
		   label bs_et = "E.-t."
		         Estimation = "Diff. de ratios"
				 bs_cv = "C.V."
				 var1 = "Num1"
				 var2 = "Dn1"
				 var3 = "Num2"
				 var4 = "Dn2"
				 n1 = "Taille Num1" 
				 n3 = "Taille Num2" 
				 valeurp = "Valeur p"
				 &ic_inf = "Borne inf. intervalle de confiance &ic%"
				 &ic_sup = "Borne sup. intervalle de confiance &ic%"
                 rep_mod = "Nombre de rpliques utilises";
		   %if (&flag_dif=0) %then %do;
	       var &classes var1 var2 var3 var4 n1 n3 Estimation z valeurp bs_et bs_cv &ic_inf &ic_sup;
	       format  bs_et &ic_inf &ic_sup Estimation valeurp 11.4 z 11.2;
		   %end;
		   %else %do;
	       var &classes var1 var2 var3 var4 n1 n3 Estimation z valeurp bs_et bs_cv &ic_inf &ic_sup rep_mod;
	       format  bs_et &ic_inf &ic_sup Estimation valeurp 11.4 z 11.2;
			 	footnote1 "------------------------------------------------------------------------------------------------------------";
				footnote2 "  Attention: Certaines rpliques n'ont pas t utilises pour le calcul de la variance. Ces rpliques   ";
                footnote3 "             ne contiennent aucune observation ayant la caractristique d'un ou des deux dnominateurs.";
				footnote4 "             Se rfrer  la colonne 'Nombre de rpliques utilises' pour connatre le nombre utilis.";
				footnote5 "------------------------------------------------------------------------------------------------------------";
		   %end;
	   run;
%end;
footnote;

%mend prntdiff;


****************;

%macro prntperc;

****************;

%if &printperc=1 %then %do;

		&by   proc sort data=allperc;
		&by     by &classes;
		&by   run;

*** Imprime les rsultats de la macro prcntle ***;

		proc print data=allperc label noobs;
		   title ;
		   title2 ;
		   title3 ;
		   title4 "Estimation de la variance  l'aide du bootstrap pour des PERCENTILES";
		   title5 "(&B rpliques bootstrap)";
		   title6 ;
		   label bs_et = "Erreur-type"
				 bs_cv = "Coeff. de variation"
				 var = "Variable"
				 perc = "Percentile (1-99)"
				 Estimation = "Valeur du percentile"
				 n = "Taille d'chantillon" 
				 &ic_inf = "Borne inf. intervalle de confiance &ic%"
				 &ic_sup = "Borne sup. intervalle de confiance &ic%"
                 rep_mod = "Nombre de rpliques utilises";
		   %if (&flag_perc=0) %then %do;
		   var &classes var n perc Estimation bs_et bs_cv &ic_inf &ic_sup;
		   format Estimation bs_et bs_cv &ic_inf &ic_sup 11.2;
		   %end;
		   %else %do;
		   var &classes var n perc Estimation bs_et bs_cv &ic_inf &ic_sup rep_mod;
		   format Estimation bs_et bs_cv &ic_inf &ic_sup 11.2;
			 	footnote1 "-------------------------------------------------------------------------------------------------------------------";
				footnote2 "  Attention: Certaines rpliques n'ont pas t utilises pour le calcul de la variance. Ces rpliques ne    ";
                footnote3 "             contiennent aucune observation ayant la caractristique d'intrt dans le domaine d'estimation.   ";
				footnote4 "             Se rfrer  la colonne 'Nombre de rpliques utilises' pour connatre le nombre utilis.";
				footnote5 "-------------------------------------------------------------------------------------------------------------------";
		   %end;
		run;
%end;
footnote;

%mend prntperc;

****************;

%macro prntreg;

****************;

%if &printreg=1 %then %do;

*** Imprime les rsultats de la macro regress ***;

		proc print data=bs_reg label noobs;
		   title ;
		   title2 ;
		   title3 ;
		   title4 "Estimation de la variance  l'aide du bootstrap pour des RGRESSIONS LINAIRES";
		   title5 "(&B rpliques bootstrap)";
		   title6 ;
		   label bs_et = "Erreur-type"
				 valeurp = "Valeur p"
				 param = "Variables indpendantes"
                 rep_mod = "Nombre de rpliques utilises";
		   by Modele;
		   %if (&flag_reg=0) %then %do;
	       var &classes param beta bs_et t valeurp ;
		   format t  11.2 beta bs_et valeurp 11.4;
		   %end;
		   %else %do;
	       var &classes param beta bs_et t valeurp rep_mod ;
		   format t  11.2 beta bs_et valeurp 11.4;
			 	footnote1 "----------------------------------------------------------------------------------------------------------------------------------";
				footnote2 "  Attention: Certaines rpliques n'ont pas t utilises pour le calcul de la variance. Ceci peut tre d au fait que     ";
                footnote3 "             certaines rpliques ne contiennent aucune observation dans le domaine d'estimation.  Ceci peut aussi tre d   ";
                footnote4 "             au fait que le model n'est pas de plein rang et que les solutions du moindre carr pour les paramtres ne sont pas ";
				footnote5 "             uniques. Se rfrer  la colonne 'Nombre de rpliques utilises' pour connatre le nombre utilis.";
				footnote6 "----------------------------------------------------------------------------------------------------------------------------------";
		   %end;
	  	run;
%end;
footnote;

%mend prntreg;

****************;

%macro prntlog;

****************;

%if &printlog=1 %then %do;

*** Imprime les rsultats de la macro logreg ***;

		proc print data=bs_reglg label noobs;
		   title ;
		   title2 ;
		   title3 ;
		   title4 "Estimation de la variance  l'aide du bootstrap pour des RGRESSIONS LOGISTIQUES";
		   title5 "(&B rpliques bootstrap)";
		   title6 ;
		   label cotes = "Rapport de cotes"
		   		 bs_et = "Erreur-type"
				 valeurp = "Valeur p"
				 &ic_inf = "Borne inf. int. de conf. (rap. de cotes) &ic%"
				 &ic_sup = "Borne sup. int. de conf. (rap. de cotes) &ic%"
				 param = "Variables indpendantes"
                 rep_mod = "Nombre de rpliques utilises";
		   by Modele;
		   %if (&flag_log=0) %then %do;
	       var  &classes param beta cotes bs_et Wald valeurp &ic_inf &ic_sup;
		   format  cotes Wald 11.2 beta bs_et valeurp &ic_inf &ic_sup  11.4;
		   %end;
		   %else %do;
	       var  &classes param beta cotes bs_et Wald valeurp &ic_inf &ic_sup rep_mod;
		   format  cotes Wald 11.2 beta bs_et valeurp &ic_inf &ic_sup  11.4;
			 	footnote1 "----------------------------------------------------------------------------------------------------------------------------------";
				footnote2 "  Attention: Certaines rpliques n'ont pas t utilises pour le calcul de la variance. Ceci peut tre d au fait que     ";
                footnote3 "             certaines rpliques ne contiennent aucune observation dans le domaine d'estimation.  Ceci peut aussi tre d   ";
                footnote4 "             au fait que le model n'est pas de plein rang et que les solutions du moindre carr pour les paramtres ne sont pas ";
				footnote5 "             uniques. Se rfrer  la colonne 'Nombre de rpliques utilises' pour connatre le nombre utilis.";
				footnote6 "----------------------------------------------------------------------------------------------------------------------------------";
		   %end;
		run;
%end;
footnote;

%mend prntlog;

****************;

%macro prntchi2;

****************;

%if &printchi2=1 %then %do;

*** Impression des rsultats de la macro CHI2 ***;

	 	   proc print data=allchi2 noobs label;
		   title ;
		   title2 ;
		   title3 ;
	       title4 "Tests d'indpendance du CHI-DEUX ajust (ajustement de Rao-Scott du second degr)";
		   title5 "(&B rpliques bootstrap)";
		   title6 ;
	       var  &classes statistic deg_freedom chi2_value p_value ;
		   format chi2_value 11.2 p_value 11.4;
		   label deg_freedom = "Deg. de libert"
		   		 chi2_value = "Valeur du Chi-deux"
				 p_value = "Valeur p"
				 statistic = "Statistique";
		  by test;
				footnote1 "---------------------------------------------------------------------------------------------------------";
				footnote2 "  Note: Un astrisque (*) dans la colonne Statistique indique que le tableau contient une ou des cellule(s) ";
				footnote3 "        avec 5 observations (non-pondres) ou moins.  Le Chi-deux peut ne pas tre un test valide dans de tels cas.";
				footnote4 "---------------------------------------------------------------------------------------------------------";
		  run;

		footnote;

		%do i=1 %to &test_num2;
		   proc print data=crosstab&i noobs label;
		   title ;
		   title2 ;
		   title3 ;
	       title4 "Tests d'indpendance du CHI-DEUX:  Statistiques varies provenant des tableaux croiss";
		   title5 "(&B rpliques bootstrap)";
		   title6 ;
		   label _label_ = "Statistique";
		   by test;
		   format total 11.4;
	 	   run;
		%end;
%end;

%mend prntchi2;

****************;

%macro output;

****************;

*** Excution des macros pour impression ***;

	%prnttot;
	%prntrat;
	%prntdiff;
	%prntperc;
	%prntreg;
	%prntlog;
	%prntchi2;

	proc datasets library=work;
	     delete Mfile bsamp version tmp;
	run;
	quit;

%mend output;



/* Fin du programme SAS MACROF_V31.SAS */
